/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP.c
Purpose : BSP for the GigaDevice GD32VF103V-EVAL.
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define LED0  (0)  // PC0
#define LED1  (2)  // PC2
#define LED2  (0)  // PE0
#define LED3  (1)  // PE1

//
// Reset and Clock Unit (RCU)
//
#define RCU_BASE_ADDR    (0x40021000u)
#define RCU_APB2EN       (*(volatile unsigned long*)(RCU_BASE_ADDR + 0x18u))  // APB2 enable register

//
// General Purpose Input Output C (GPIOC)
//
#define GPIOC_BASE_ADDR  (0x40011000u)
#define GPIOC_CTL0       (*(volatile unsigned long*)(GPIOC_BASE_ADDR + 0x00u))  // Port control register 0
#define GPIOC_OCTL       (*(volatile unsigned long*)(GPIOC_BASE_ADDR + 0x0Cu))  // Port output control register
#define GPIOC_BOP        (*(volatile unsigned long*)(GPIOC_BASE_ADDR + 0x10u))  // Port bit operate register

//
// General Purpose Input Output E (GPIOE)
//
#define GPIOE_BASE_ADDR  (0x40011800u)
#define GPIOE_CTL0       (*(volatile unsigned long*)(GPIOE_BASE_ADDR + 0x00u))  // Port control register 0
#define GPIOE_OCTL       (*(volatile unsigned long*)(GPIOE_BASE_ADDR + 0x0Cu))  // Port output control register
#define GPIOE_BOP        (*(volatile unsigned long*)(GPIOE_BASE_ADDR + 0x10u))  // Port bit operate register

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Enable clock for GPIOC and GPIOE
  //
  RCU_APB2EN |= (1u << 4)
             |  (1u << 6);
  //
  // Turn LEDs off
  //
  GPIOC_BOP = (1u << (LED0 + 16));
  GPIOC_BOP = (1u << (LED1 + 16));
  GPIOE_BOP = (1u << (LED2 + 16));
  GPIOE_BOP = (1u << (LED3 + 16));
  //
  // Clear previous GPIO setting and set them to 50MHz output with push-pull
  //
  GPIOC_CTL0 &= ~((0xFu << (LED0 * 4))
             |    (0xFu << (LED1 * 4)));
  GPIOC_CTL0 |=   (0x3u << (LED0 * 4))
             |    (0x3u << (LED1 * 4));
  GPIOE_CTL0 &= ~((0xFu << (LED2 * 4))
             |    (0xFu << (LED3 * 4)));
  GPIOE_CTL0 |=   (0x3u << (LED2 * 4))
             |    (0x3u << (LED3 * 4));
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  switch (Index) {
  case 0:
    GPIOC_BOP = (1u << LED0);
    break;
  case 1:
    GPIOC_BOP = (1u << LED1);
    break;
  case 2:
    GPIOE_BOP = (1u << LED2);
    break;
  case 3:
    GPIOE_BOP = (1u << LED3);
    break;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  switch (Index) {
  case 0:
    GPIOC_BOP = (1u << (LED0 + 16));
    break;
  case 1:
    GPIOC_BOP = (1u << (LED1 + 16));
    break;
  case 2:
    GPIOE_BOP = (1u << (LED2 + 16));
    break;
  case 3:
    GPIOE_BOP = (1u << (LED3 + 16));
    break;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  switch (Index) {
  case 0:
    GPIOC_OCTL ^= (1u << LED0);
    break;
  case 1:
    GPIOC_OCTL ^= (1u << LED1);
    break;
  case 2:
    GPIOE_OCTL ^= (1u << LED2);
    break;
  case 3:
    GPIOE_OCTL ^= (1u << LED3);
    break;
  }
}

/*************************** End of file ****************************/
