/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : entry.s
Purpose : Entry point and vector table for the GD32VF103.

Additional information:
  [1] The RISC-V Instruction Set Manual Volume II: Privileged Architecture
      ...
      3.1.7 Machine Trap-Vector Base-Address Register (mtvec)
        ...
        When vectored interrupts are enabled, interrupt cause 0, which corresponds to user-mode software interrupts,
        are vectored to the same location as synchronous exceptions. This ambiguity does not arise in practice,
        since user-mode software interrupts are either disabled or delegated to user mode.
        ...
*/

/*********************************************************************
*
*       Defines, fixed
*
**********************************************************************
*/

#if (__riscv_xlen == 64)
  #define STORE     sd
  #define LOAD      ld
  #define REGBYTES   8
#else
  #define STORE     sw
  #define LOAD      lw
  #define REGBYTES   4
#endif

/*********************************************************************
*
*       MACROS
*
**********************************************************************
*/

/*********************************************************************
*
*     Publics
*
**********************************************************************
*/
        PUBLIC  vectors

        PUBWEAK ISR_U_Software
        PUBWEAK ISR_S_Software
        PUBWEAK ISR_U_Timer
        PUBWEAK ISR_S_Timer
        PUBWEAK ISR_U_External
        PUBWEAK ISR_S_External
        PUBWEAK ISR_M_External
        PUBWEAK eclic_msip_handler
        PUBWEAK eclic_mtip_handler
        PUBWEAK eclic_bwei_handler
        PUBWEAK eclic_pmovi_handler
        PUBWEAK WWDGT_IRQHandler
        PUBWEAK LVD_IRQHandler
        PUBWEAK TAMPER_IRQHandler
        PUBWEAK RTC_IRQHandler
        PUBWEAK FMC_IRQHandler
        PUBWEAK RCU_IRQHandler
        PUBWEAK EXTI0_IRQHandler
        PUBWEAK EXTI1_IRQHandler
        PUBWEAK EXTI2_IRQHandler
        PUBWEAK EXTI3_IRQHandler
        PUBWEAK EXTI4_IRQHandler
        PUBWEAK DMA0_Channel0_IRQHandler
        PUBWEAK DMA0_Channel1_IRQHandler
        PUBWEAK DMA0_Channel2_IRQHandler
        PUBWEAK DMA0_Channel3_IRQHandler
        PUBWEAK DMA0_Channel4_IRQHandler
        PUBWEAK DMA0_Channel5_IRQHandler
        PUBWEAK DMA0_Channel6_IRQHandler
        PUBWEAK ADC0_1_IRQHandler
        PUBWEAK CAN0_TX_IRQHandler
        PUBWEAK CAN0_RX0_IRQHandler
        PUBWEAK CAN0_RX1_IRQHandler
        PUBWEAK CAN0_EWMC_IRQHandler
        PUBWEAK EXTI5_9_IRQHandler
        PUBWEAK TIMER0_BRK_IRQHandler
        PUBWEAK TIMER0_UP_IRQHandler
        PUBWEAK TIMER0_TRG_CMT_IRQHandler
        PUBWEAK TIMER0_Channel_IRQHandler
        PUBWEAK TIMER1_IRQHandler
        PUBWEAK TIMER2_IRQHandler
        PUBWEAK TIMER3_IRQHandler
        PUBWEAK I2C0_EV_IRQHandler
        PUBWEAK I2C0_ER_IRQHandler
        PUBWEAK I2C1_EV_IRQHandler
        PUBWEAK I2C1_ER_IRQHandler
        PUBWEAK SPI0_IRQHandler
        PUBWEAK SPI1_IRQHandler
        PUBWEAK USART0_IRQHandler
        PUBWEAK USART1_IRQHandler
        PUBWEAK USART2_IRQHandler
        PUBWEAK EXTI10_15_IRQHandler
        PUBWEAK RTC_Alarm_IRQHandler
        PUBWEAK USBFS_WKUP_IRQHandler
        PUBWEAK EXMC_IRQHandler
        PUBWEAK TIMER4_IRQHandler
        PUBWEAK SPI2_IRQHandler
        PUBWEAK UART3_IRQHandler
        PUBWEAK UART4_IRQHandler
        PUBWEAK TIMER5_IRQHandler
        PUBWEAK TIMER6_IRQHandler
        PUBWEAK DMA1_Channel0_IRQHandler
        PUBWEAK DMA1_Channel1_IRQHandler
        PUBWEAK DMA1_Channel2_IRQHandler
        PUBWEAK DMA1_Channel3_IRQHandler
        PUBWEAK DMA1_Channel4_IRQHandler
        PUBWEAK CAN1_TX_IRQHandler
        PUBWEAK CAN1_RX0_IRQHandler
        PUBWEAK CAN1_RX1_IRQHandler
        PUBWEAK CAN1_EWMC_IRQHandler
        PUBWEAK USBFS_IRQHandler

/*********************************************************************
*
*     Externals, code
*
**********************************************************************
*/
        EXTERN  OS_TrapHandler
        EXTERN  ISR_M_Software
        EXTERN  ISR_M_Timer

/*********************************************************************
*
*       Vector table
*/
        SECTION `.mintvec`:CODE:NOROOT(2)

        ALIGN 8
vectors:
        dc32 OS_TrapHandler  // This vector is ambiguously used for user mode software interrupts as well (see [1])
        dc32 ISR_S_Software
        dc32 Undefined       // reserved, previously intended for Hypervisor mode
        dc32 ISR_M_Software
        dc32 ISR_U_Timer
        dc32 ISR_S_Timer
        dc32 Undefined       // reserved, previously intended for Hypervisor mode
        dc32 ISR_M_Timer
        dc32 Undefined       // reserved, originally intended for user external
        dc32 Undefined       // reserved, originally intended for software external
        dc32 Undefined       // reserved, previously intended for Hypervisor mode
        dc32 Undefined       // reserved, originally intended for machine external
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 eclic_bwei_handler
        dc32 eclic_pmovi_handler
        dc32 WWDGT_IRQHandler
        dc32 LVD_IRQHandler
        dc32 TAMPER_IRQHandler
        dc32 RTC_IRQHandler
        dc32 FMC_IRQHandler
        dc32 RCU_IRQHandler
        dc32 EXTI0_IRQHandler
        dc32 EXTI1_IRQHandler
        dc32 EXTI2_IRQHandler
        dc32 EXTI3_IRQHandler
        dc32 EXTI4_IRQHandler
        dc32 DMA0_Channel0_IRQHandler
        dc32 DMA0_Channel1_IRQHandler
        dc32 DMA0_Channel2_IRQHandler
        dc32 DMA0_Channel3_IRQHandler
        dc32 DMA0_Channel4_IRQHandler
        dc32 DMA0_Channel5_IRQHandler
        dc32 DMA0_Channel6_IRQHandler
        dc32 ADC0_1_IRQHandler
        dc32 CAN0_TX_IRQHandler
        dc32 CAN0_RX0_IRQHandler
        dc32 CAN0_RX1_IRQHandler
        dc32 CAN0_EWMC_IRQHandler
        dc32 EXTI5_9_IRQHandler
        dc32 TIMER0_BRK_IRQHandler
        dc32 TIMER0_UP_IRQHandler
        dc32 TIMER0_TRG_CMT_IRQHandler
        dc32 TIMER0_Channel_IRQHandler
        dc32 TIMER1_IRQHandler
        dc32 TIMER2_IRQHandler
        dc32 TIMER3_IRQHandler
        dc32 I2C0_EV_IRQHandler
        dc32 I2C0_ER_IRQHandler
        dc32 I2C1_EV_IRQHandler
        dc32 I2C1_ER_IRQHandler
        dc32 SPI0_IRQHandler
        dc32 SPI1_IRQHandler
        dc32 USART0_IRQHandler
        dc32 USART1_IRQHandler
        dc32 USART2_IRQHandler
        dc32 EXTI10_15_IRQHandler
        dc32 RTC_Alarm_IRQHandler
        dc32 USBFS_WKUP_IRQHandler
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 EXMC_IRQHandler
        dc32 Undefined       // reserved
        dc32 TIMER4_IRQHandler
        dc32 SPI2_IRQHandler
        dc32 UART3_IRQHandler
        dc32 UART4_IRQHandler
        dc32 TIMER5_IRQHandler
        dc32 TIMER6_IRQHandler
        dc32 DMA1_Channel0_IRQHandler
        dc32 DMA1_Channel1_IRQHandler
        dc32 DMA1_Channel2_IRQHandler
        dc32 DMA1_Channel3_IRQHandler
        dc32 DMA1_Channel4_IRQHandler
        dc32 Undefined       // reserved
        dc32 Undefined       // reserved
        dc32 CAN1_TX_IRQHandler
        dc32 CAN1_RX0_IRQHandler
        dc32 CAN1_RX1_IRQHandler
        dc32 CAN1_EWMC_IRQHandler
        dc32 USBFS_IRQHandler

/*********************************************************************
*
*       Weak high-level IRQ handler. Can be overwritten in "C".
*/
        ISR_U_Software:
        ISR_S_Software:
        ISR_U_Timer:
        ISR_S_Timer:
        ISR_U_External:
        ISR_S_External:
        ISR_M_External:
        eclic_msip_handler:
        eclic_mtip_handler:
        eclic_bwei_handler:
        eclic_pmovi_handler:
        WWDGT_IRQHandler:
        LVD_IRQHandler:
        TAMPER_IRQHandler:
        RTC_IRQHandler:
        FMC_IRQHandler:
        RCU_IRQHandler:
        EXTI0_IRQHandler:
        EXTI1_IRQHandler:
        EXTI2_IRQHandler:
        EXTI3_IRQHandler:
        EXTI4_IRQHandler:
        DMA0_Channel0_IRQHandler:
        DMA0_Channel1_IRQHandler:
        DMA0_Channel2_IRQHandler:
        DMA0_Channel3_IRQHandler:
        DMA0_Channel4_IRQHandler:
        DMA0_Channel5_IRQHandler:
        DMA0_Channel6_IRQHandler:
        ADC0_1_IRQHandler:
        CAN0_TX_IRQHandler:
        CAN0_RX0_IRQHandler:
        CAN0_RX1_IRQHandler:
        CAN0_EWMC_IRQHandler:
        EXTI5_9_IRQHandler:
        TIMER0_BRK_IRQHandler:
        TIMER0_UP_IRQHandler:
        TIMER0_TRG_CMT_IRQHandler:
        TIMER0_Channel_IRQHandler:
        TIMER1_IRQHandler:
        TIMER2_IRQHandler:
        TIMER3_IRQHandler:
        I2C0_EV_IRQHandler:
        I2C0_ER_IRQHandler:
        I2C1_EV_IRQHandler:
        I2C1_ER_IRQHandler:
        SPI0_IRQHandler:
        SPI1_IRQHandler:
        USART0_IRQHandler:
        USART1_IRQHandler:
        USART2_IRQHandler:
        EXTI10_15_IRQHandler:
        RTC_Alarm_IRQHandler:
        USBFS_WKUP_IRQHandler:
        EXMC_IRQHandler:
        TIMER4_IRQHandler:
        SPI2_IRQHandler:
        UART3_IRQHandler:
        UART4_IRQHandler:
        TIMER5_IRQHandler:
        TIMER6_IRQHandler:
        DMA1_Channel0_IRQHandler:
        DMA1_Channel1_IRQHandler:
        DMA1_Channel2_IRQHandler:
        DMA1_Channel3_IRQHandler:
        DMA1_Channel4_IRQHandler:
        CAN1_TX_IRQHandler:
        CAN1_RX0_IRQHandler:
        CAN1_RX1_IRQHandler:
        CAN1_EWMC_IRQHandler:
        USBFS_IRQHandler:
Undefined:  j       Undefined
/********************************************************************/

        END

/*************************** End of file ****************************/
