/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP.c
Purpose : BSP for Freedom E31 Coreplex IP Evaluation Arty FPGA Dev Kit
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define NUMLEDS          (3)        // We use LD0 (RGB-LED)
#define LD0_RED_MASK     (1u << 0)
#define LD0_GREEN_MASK   (1u << 1)
#define LD0_BLUE_MASK    (1u << 2)

#define GPIO_BASE_ADDR   (0x20002000u)
#define GPIO_OUTPUT_EN   (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0x8u))
#define GPIO_OUTPUT_VAL  (*(volatile unsigned int*)(GPIO_BASE_ADDR + 0xCu))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  GPIO_OUTPUT_EN  |=  (LD0_RED_MASK | LD0_GREEN_MASK | LD0_BLUE_MASK);  // Enable GPIO output
  GPIO_OUTPUT_VAL &= ~(LD0_RED_MASK | LD0_GREEN_MASK | LD0_BLUE_MASK);  // Initially clear LD0
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < NUMLEDS) {
    GPIO_OUTPUT_VAL |= (1u << Index);  // Set LED
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < NUMLEDS) {
    GPIO_OUTPUT_VAL &= ~(1u << Index);  // Clear LED
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index < NUMLEDS) {
    GPIO_OUTPUT_VAL ^= (1u << Index);  // Invert LED
  }
}

/*************************** End of file ****************************/
